@@ -1,4 +1,4 @@
-from pangres.core import upsert
+from pangres.core import aupsert, upsert
 from pangres.utils import adjust_chunksize, fix_psycopg2_bad_cols
 from pangres.examples import DocsExampleTable
 from pangres._version import __version__
